@@ -7,8 +7,10 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "api/api_who_reacted.h"
 
+#include "api/api_global_privacy.h"
 #include "history/history_item.h"
 #include "history/history.h"
+#include "data/stickers/data_custom_emoji.h"
 #include "data/data_peer.h"
 #include "data/data_chat.h"
 #include "data/data_channel.h"
@@ -17,56 +19,58 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_changes.h"
 #include "data/data_session.h"
 #include "data/data_media_types.h"
+#include "data/data_message_reaction_id.h"
+#include "data/data_peer_values.h"
+#include "lang/lang_keys.h"
 #include "main/main_app_config.h"
 #include "main/main_session.h"
-#include "main/main_account.h"
 #include "base/unixtime.h"
 #include "base/weak_ptr.h"
 #include "ui/controls/who_reacted_context_action.h"
 #include "apiwrap.h"
 #include "styles/style_chat.h"
+#include "styles/style_chat_helpers.h"
 
 namespace Api {
 namespace {
 
 constexpr auto kContextReactionsLimit = 50;
 
+using Data::ReactionId;
+using WhoReadState = Ui::WhoReadState;
+
 struct Peers {
-	std::vector<PeerId> list;
-	bool unknown = false;
+	std::vector<WhoReadPeer> list;
+	WhoReadState state = WhoReadState::Empty;
+
+	friend inline bool operator==(
+		const Peers &a,
+		const Peers &b) noexcept = default;
 };
-inline bool operator==(const Peers &a, const Peers &b) noexcept {
-	return (a.list == b.list) && (a.unknown == b.unknown);
-}
 
 struct PeerWithReaction {
-	PeerId peer = 0;
-	QString reaction;
-};
-inline bool operator==(
+	WhoReadPeer peerWithDate;
+	ReactionId reaction;
+
+	friend inline bool operator==(
 		const PeerWithReaction &a,
-		const PeerWithReaction &b) noexcept {
-	return (a.peer == b.peer) && (a.reaction == b.reaction);
-}
+		const PeerWithReaction &b) noexcept = default;
+};
 
 struct PeersWithReactions {
 	std::vector<PeerWithReaction> list;
-	std::vector<PeerId> read;
+	std::vector<WhoReadPeer> read;
 	int fullReactionsCount = 0;
-	bool unknown = false;
-};
-inline bool operator==(
+	WhoReadState state = WhoReadState::Empty;
+
+	friend inline bool operator==(
 		const PeersWithReactions &a,
-		const PeersWithReactions &b) noexcept {
-	return (a.fullReactionsCount == b.fullReactionsCount)
-		&& (a.list == b.list)
-		&& (a.read == b.read)
-		&& (a.unknown == b.unknown);
-}
+		const PeersWithReactions &b) noexcept = default;
+};
 
 struct CachedRead {
 	CachedRead()
-	: data(Peers{ .unknown = true }) {
+	: data(Peers{ .state = WhoReadState::Unknown }) {
 	}
 	rpl::variable<Peers> data;
 	mtpRequestId requestId = 0;
@@ -74,7 +78,7 @@ struct CachedRead {
 
 struct CachedReacted {
 	CachedReacted()
-	: data(PeersWithReactions{ .unknown = true }) {
+	: data(PeersWithReactions{ .state = WhoReadState::Unknown }) {
 	}
 	rpl::variable<PeersWithReactions> data;
 	mtpRequestId requestId = 0;
@@ -84,7 +88,7 @@ struct Context {
 	base::flat_map<not_null<HistoryItem*>, CachedRead> cachedRead;
 	base::flat_map<
 		not_null<HistoryItem*>,
-		base::flat_map<QString, CachedReacted>> cachedReacted;
+		base::flat_map<ReactionId, CachedReacted>> cachedReacted;
 	base::flat_map<not_null<Main::Session*>, rpl::lifetime> subscriptions;
 
 	[[nodiscard]] CachedRead &cacheRead(not_null<HistoryItem*> item) {
@@ -97,7 +101,7 @@ struct Context {
 
 	[[nodiscard]] CachedReacted &cacheReacted(
 			not_null<HistoryItem*> item,
-			const QString &reaction) {
+			const ReactionId &reaction) {
 		auto &map = cachedReacted[item];
 		const auto i = map.find(reaction);
 		if (i != end(map)) {
@@ -109,8 +113,10 @@ struct Context {
 
 struct Userpic {
 	not_null<PeerData*> peer;
-	QString reaction;
-	mutable std::shared_ptr<Data::CloudImageView> view;
+	TimeId date = 0;
+	bool dateReacted = false;
+	QString customEntityData;
+	mutable Ui::PeerUserpicView view;
 	mutable InMemoryKey uniqueKey;
 };
 
@@ -182,12 +188,36 @@ struct State {
 			context->cachedReacted.erase(j);
 		}
 	}, context->subscriptions[session]);
+	Data::AmPremiumValue(
+		session
+	) | rpl::skip(1) | rpl::filter(
+		rpl::mappers::_1
+	) | rpl::start_with_next([=] {
+		for (auto &[item, cache] : context->cachedRead) {
+			if (cache.data.current().state == Ui::WhoReadState::MyHidden) {
+				cache.data = Peers{ .state = Ui::WhoReadState::Unknown };
+			}
+		}
+	}, context->subscriptions[session]);
+	session->api().globalPrivacy().hideReadTime(
+	) | rpl::skip(1) | rpl::filter(
+		!rpl::mappers::_1
+	) | rpl::start_with_next([=] {
+		for (auto &[item, cache] : context->cachedRead) {
+			if (cache.data.current().state == Ui::WhoReadState::MyHidden) {
+				cache.data = Peers{ .state = Ui::WhoReadState::Unknown };
+			}
+		}
+	}, context->subscriptions[session]);
 	return context;
 }
 
 [[nodiscard]] QImage GenerateUserpic(Userpic &userpic, int size) {
 	size *= style::DevicePixelRatio();
-	auto result = userpic.peer->generateUserpicImage(userpic.view, size);
+	auto result = PeerData::GenerateUserpicImage(
+		userpic.peer,
+		userpic.view,
+		size);
 	result.setDevicePixelRatio(style::DevicePixelRatio());
 	return result;
 }
@@ -210,34 +240,68 @@ struct State {
 [[nodiscard]] rpl::producer<Peers> WhoReadIds(
 		not_null<HistoryItem*> item,
 		not_null<QWidget*> context) {
-	auto weak = QPointer<QWidget>(context.get());
+	auto weak = base::make_weak(context);
 	const auto session = &item->history()->session();
 	return [=](auto consumer) {
 		if (!weak) {
 			return rpl::lifetime();
 		}
-		const auto context = PreparedContextAt(weak.data(), session);
+		const auto context = PreparedContextAt(weak.get(), session);
 		auto &entry = context->cacheRead(item);
-		if (!entry.requestId) {
+		if (entry.requestId) {
+		} else if (const auto user = item->history()->peer->asUser()) {
+			entry.requestId = session->api().request(
+				MTPmessages_GetOutboxReadDate(
+					user->input,
+					MTP_int(item->id)
+				)
+			).done([=](const MTPOutboxReadDate &result) {
+				const auto &data = result.data();
+				auto &entry = context->cacheRead(item);
+				entry.requestId = 0;
+				auto parsed = Peers();
+				parsed.list.push_back({
+					.peer = user->id,
+					.date = data.vdate().v,
+				});
+				entry.data = std::move(parsed);
+			}).fail([=](const MTP::Error &error) {
+				auto &entry = context->cacheRead(item);
+				entry.requestId = 0;
+				if (entry.data.current().state == WhoReadState::Unknown) {
+					const auto &text = error.type();
+					entry.data = (text == u"YOUR_PRIVACY_RESTRICTED"_q)
+						? Peers{ .state = WhoReadState::MyHidden }
+						: (text == u"USER_PRIVACY_RESTRICTED"_q)
+						? Peers{ .state = WhoReadState::HisHidden }
+						: (text == u"MESSAGE_TOO_OLD"_q)
+						? Peers{ .state = WhoReadState::TooOld }
+						: Peers{ .state = WhoReadState::Empty };
+				}
+			}).send();
+		} else {
 			entry.requestId = session->api().request(
 				MTPmessages_GetMessageReadParticipants(
 					item->history()->peer->input,
 					MTP_int(item->id)
 				)
-			).done([=](const MTPVector<MTPlong> &result) {
+			).done([=](const MTPVector<MTPReadParticipantDate> &result) {
 				auto &entry = context->cacheRead(item);
 				entry.requestId = 0;
 				auto parsed = Peers();
 				parsed.list.reserve(result.v.size());
 				for (const auto &id : result.v) {
-					parsed.list.push_back(UserId(id));
+					parsed.list.push_back({
+						.peer = UserId(id.data().vuser_id()),
+						.date = id.data().vdate().v,
+					});
 				}
 				entry.data = std::move(parsed);
 			}).fail([=] {
 				auto &entry = context->cacheRead(item);
 				entry.requestId = 0;
-				if (entry.data.current().unknown) {
-					entry.data = Peers();
+				if (entry.data.current().state == WhoReadState::Unknown) {
+					entry.data = Peers{ .state = WhoReadState::Empty };
 				}
 			}).send();
 		}
@@ -248,10 +312,10 @@ struct State {
 [[nodiscard]] PeersWithReactions WithEmptyReactions(
 		Peers &&peers) {
 	auto result = PeersWithReactions{
-		.list = peers.list | ranges::views::transform([](PeerId peer) {
-			return PeerWithReaction{.peer = peer };
+		.list = peers.list | ranges::views::transform([](WhoReadPeer peer) {
+			return PeerWithReaction{ .peerWithDate = peer };
 		}) | ranges::to_vector,
-		.unknown = peers.unknown,
+		.state = peers.state,
 	};
 	result.read = std::move(peers.list);
 	return result;
@@ -259,26 +323,26 @@ struct State {
 
 [[nodiscard]] rpl::producer<PeersWithReactions> WhoReactedIds(
 		not_null<HistoryItem*> item,
-		const QString &reaction,
+		const ReactionId &reaction,
 		not_null<QWidget*> context) {
-	auto weak = QPointer<QWidget>(context.get());
+	auto weak = base::make_weak(context);
 	const auto session = &item->history()->session();
 	return [=](auto consumer) {
 		if (!weak) {
 			return rpl::lifetime();
 		}
-		const auto context = PreparedContextAt(weak.data(), session);
+		const auto context = PreparedContextAt(weak.get(), session);
 		auto &entry = context->cacheReacted(item, reaction);
 		if (!entry.requestId) {
 			using Flag = MTPmessages_GetMessageReactionsList::Flag;
 			entry.requestId = session->api().request(
 				MTPmessages_GetMessageReactionsList(
-					MTP_flags(reaction.isEmpty()
+					MTP_flags(reaction.empty()
 						? Flag(0)
 						: Flag::f_reaction),
 					item->history()->peer->input,
 					MTP_int(item->id),
-					MTP_string(reaction),
+					ReactionToMTP(reaction),
 					MTPstring(), // offset
 					MTP_int(kContextReactionsLimit)
 				)
@@ -296,11 +360,15 @@ struct State {
 					};
 					parsed.list.reserve(data.vreactions().v.size());
 					for (const auto &vote : data.vreactions().v) {
-						vote.match([&](const auto &data) {
-							parsed.list.push_back(PeerWithReaction{
+						const auto &data = vote.data();
+						parsed.list.push_back(PeerWithReaction{
+							.peerWithDate = {
 								.peer = peerFromMTP(data.vpeer_id()),
-								.reaction = qs(data.vreaction()),
-							});
+								.date = data.vdate().v,
+								.dateReacted = true,
+							},
+							.reaction = Data::ReactionFromMTP(
+								data.vreaction()),
 						});
 					}
 					entry.data = std::move(parsed);
@@ -308,8 +376,10 @@ struct State {
 			}).fail([=] {
 				auto &entry = context->cacheReacted(item, reaction);
 				entry.requestId = 0;
-				if (entry.data.current().unknown) {
-					entry.data = PeersWithReactions();
+				if (entry.data.current().state == WhoReadState::Unknown) {
+					entry.data = PeersWithReactions{
+						.state = WhoReadState::Empty,
+					};
 				}
 			}).send();
 		}
@@ -322,16 +392,25 @@ struct State {
 	not_null<QWidget*> context)
 -> rpl::producer<PeersWithReactions> {
 	return rpl::combine(
-		WhoReactedIds(item, QString(), context),
+		WhoReactedIds(item, {}, context),
 		WhoReadIds(item, context)
 	) | rpl::map([=](PeersWithReactions &&reacted, Peers &&read) {
-		if (reacted.unknown || read.unknown) {
-			return PeersWithReactions{ .unknown = true };
+		if (reacted.state == WhoReadState::Unknown
+			|| read.state == WhoReadState::Unknown) {
+			return PeersWithReactions{ .state = WhoReadState::Unknown};
 		}
 		auto &list = reacted.list;
-		for (const auto &peer : read.list) {
-			if (!ranges::contains(list, peer, &PeerWithReaction::peer)) {
-				list.push_back({ .peer = peer });
+		for (const auto &peerWithDate : read.list) {
+			const auto i = ranges::find(
+				list,
+				peerWithDate.peer,
+				[](const PeerWithReaction &p) {
+					return p.peerWithDate.peer; });
+			if (i == end(list)) {
+				list.push_back({ .peerWithDate = peerWithDate });
+			} else if (!i->peerWithDate.date) {
+				i->peerWithDate.date = peerWithDate.date;
+				i->peerWithDate.dateReacted = peerWithDate.dateReacted;
 			}
 		}
 		reacted.read = std::move(read.list);
@@ -347,13 +426,17 @@ bool UpdateUserpics(
 
 	struct ResolvedPeer {
 		PeerData *peer = nullptr;
-		QString reaction;
+		TimeId date = 0;
+		bool dateReacted = false;
+		ReactionId reaction;
 	};
 	const auto peers = ranges::views::all(
 		ids
 	) | ranges::views::transform([&](PeerWithReaction id) {
 		return ResolvedPeer{
-			.peer = owner.peerLoaded(id.peer),
+			.peer = owner.peerLoaded(id.peerWithDate.peer),
+			.date = id.peerWithDate.date,
+			.dateReacted = id.peerWithDate.dateReacted,
 			.reaction = id.reaction,
 		};
 	}) | ranges::views::filter([](ResolvedPeer resolved) {
@@ -364,8 +447,8 @@ bool UpdateUserpics(
 		state->userpics,
 		peers,
 		ranges::equal_to(),
-		&Userpic::peer,
-		[](const ResolvedPeer &r) { return not_null{ r.peer }; });
+		[](const Userpic &u) { return std::pair(u.peer.get(), u.date); },
+		[](const ResolvedPeer &r) { return std::pair(r.peer, r.date); });
 	if (same) {
 		return false;
 	}
@@ -373,17 +456,20 @@ bool UpdateUserpics(
 	auto now = std::vector<Userpic>();
 	for (const auto &resolved : peers) {
 		const auto peer = not_null{ resolved.peer };
-		if (ranges::contains(now, peer, &Userpic::peer)) {
-			continue;
-		}
+		const auto &data = ReactionEntityData(resolved.reaction);
 		const auto i = ranges::find(was, peer, &Userpic::peer);
-		if (i != end(was)) {
+		if (i != end(was) && i->view.cloud) {
+			i->date = resolved.date;
+			i->dateReacted = resolved.dateReacted;
 			now.push_back(std::move(*i));
+			now.back().customEntityData = data;
 			continue;
 		}
 		now.push_back(Userpic{
 			.peer = peer,
-			.reaction = resolved.reaction,
+			.date = resolved.date,
+			.dateReacted = resolved.dateReacted,
+			.customEntityData = data,
 		});
 		auto &userpic = now.back();
 		userpic.uniqueKey = peer->userpicUniqueKey(userpic.view);
@@ -418,21 +504,27 @@ void RegenerateUserpics(not_null<State*> state, int small, int large) {
 }
 
 void RegenerateParticipants(not_null<State*> state, int small, int large) {
+	const auto currentDate = QDateTime::currentDateTime();
 	auto old = base::take(state->current.participants);
 	auto &now = state->current.participants;
 	now.reserve(state->userpics.size());
 	for (auto &userpic : state->userpics) {
 		const auto peer = userpic.peer;
+		const auto date = userpic.date;
 		const auto id = peer->id.value;
 		const auto was = ranges::find(old, id, &Ui::WhoReadParticipant::id);
 		if (was != end(old)) {
-			was->name = peer->name;
+			was->name = peer->name();
+			was->date = FormatReadDate(date, currentDate);
+			was->dateReacted = userpic.dateReacted;
 			now.push_back(std::move(*was));
 			continue;
 		}
 		now.push_back({
-			.name = peer->name,
-			.reaction = userpic.reaction,
+			.name = peer->name(),
+			.date = FormatReadDate(date, currentDate),
+			.dateReacted = userpic.dateReacted,
+			.customEntityData = userpic.customEntityData,
 			.userpicLarge = GenerateUserpic(userpic, large),
 			.userpicKey = userpic.uniqueKey,
 			.id = id,
@@ -446,7 +538,7 @@ void RegenerateParticipants(not_null<State*> state, int small, int large) {
 
 rpl::producer<Ui::WhoReadContent> WhoReacted(
 		not_null<HistoryItem*> item,
-		const QString &reaction,
+		const ReactionId &reaction,
 		not_null<QWidget*> context,
 		const style::WhoRead &st,
 		std::shared_ptr<WhoReadList> whoReadIds) {
@@ -455,7 +547,7 @@ rpl::producer<Ui::WhoReadContent> WhoReacted(
 	return [=](auto consumer) {
 		auto lifetime = rpl::lifetime();
 
-		const auto resolveWhoRead = reaction.isEmpty()
+		const auto resolveWhoRead = reaction.empty()
 			&& WhoReadExists(item);
 
 		const auto state = lifetime.make_state<State>();
@@ -463,7 +555,7 @@ rpl::producer<Ui::WhoReadContent> WhoReacted(
 			consumer.put_next_copy(state->current);
 		};
 
-		const auto resolveWhoReacted = !reaction.isEmpty()
+		const auto resolveWhoReacted = !reaction.empty()
 			|| item->canViewReactions();
 		auto idsWithReactions = (resolveWhoRead && resolveWhoReacted)
 			? WhoReadOrReactedIds(item, context)
@@ -475,42 +567,51 @@ rpl::producer<Ui::WhoReadContent> WhoReacted(
 			: Ui::WhoReadType::Reacted;
 		if (resolveWhoReacted) {
 			const auto &list = item->reactions();
-			state->current.fullReactionsCount = reaction.isEmpty()
-				? ranges::accumulate(
+			state->current.fullReactionsCount = [&] {
+				if (reaction.empty()) {
+					return ranges::accumulate(
+						list,
+						0,
+						ranges::plus{},
+						&Data::MessageReaction::count);
+				}
+				const auto i = ranges::find(
 					list,
-					0,
-					ranges::plus{},
-					[](const auto &pair) { return pair.second; })
-				: list.contains(reaction)
-				? list.find(reaction)->second
-				: 0;
-
-			// #TODO reactions
-			state->current.singleReaction = !reaction.isEmpty()
+					reaction,
+					&Data::MessageReaction::id);
+				return (i != end(list)) ? i->count : 0;
+			}();
+			state->current.singleCustomEntityData = ReactionEntityData(
+				!reaction.empty()
 				? reaction
 				: (list.size() == 1)
-				? list.front().first
-				: QString();
+				? list.front().id
+				: ReactionId());
 		}
 		std::move(
 			idsWithReactions
 		) | rpl::start_with_next([=](PeersWithReactions &&peers) {
-			if (peers.unknown) {
+			if (peers.state == WhoReadState::Unknown) {
 				state->userpics.clear();
 				consumer.put_next(Ui::WhoReadContent{
 					.type = state->current.type,
 					.fullReactionsCount = state->current.fullReactionsCount,
 					.fullReadCount = state->current.fullReadCount,
-					.unknown = true,
+					.state = WhoReadState::Unknown,
 				});
 				return;
 			}
+			state->current.state = peers.state;
 			state->current.fullReadCount = int(peers.read.size());
 			state->current.fullReactionsCount = peers.fullReactionsCount;
 			if (whoReadIds) {
-				whoReadIds->list = (peers.read.size() > peers.list.size())
+				const auto reacted = peers.list.size() - ranges::count(
+					peers.list,
+					ReactionId(),
+					&PeerWithReaction::reaction);
+				whoReadIds->list = (peers.read.size() > reacted)
 					? std::move(peers.read)
-					: std::vector<PeerId>();
+					: std::vector<WhoReadPeer>();
 			}
 			if (UpdateUserpics(state, item, peers.list)) {
 				RegenerateParticipants(state, small, large);
@@ -544,34 +645,87 @@ rpl::producer<Ui::WhoReadContent> WhoReacted(
 
 } // namespace
 
+QString FormatReadDate(TimeId date, const QDateTime &now) {
+	if (!date) {
+		return {};
+	}
+	const auto parsed = base::unixtime::parse(date);
+	const auto readDate = parsed.date();
+	const auto nowDate = now.date();
+	if (readDate == nowDate) {
+		return tr::lng_mediaview_today(
+			tr::now,
+			lt_time,
+			QLocale().toString(parsed.time(), QLocale::ShortFormat));
+	} else if (readDate.addDays(1) == nowDate) {
+		return tr::lng_mediaview_yesterday(
+			tr::now,
+			lt_time,
+			QLocale().toString(parsed.time(), QLocale::ShortFormat));
+	}
+	return tr::lng_mediaview_date_time(
+		tr::now,
+		lt_date,
+		tr::lng_month_day(
+			tr::now,
+			lt_month,
+			Lang::MonthDay(readDate.month())(tr::now),
+			lt_day,
+			QString::number(readDate.day())),
+		lt_time,
+		QLocale().toString(parsed.time(), QLocale::ShortFormat));
+}
+
 bool WhoReadExists(not_null<HistoryItem*> item) {
 	if (!item->out()) {
 		return false;
 	}
 	const auto type = DetectSeenType(item);
+	const auto thread = item->topic()
+		? (Data::Thread*)item->topic()
+		: item->history();
 	const auto unseen = (type == Ui::WhoReadType::Seen)
-		? item->unread()
+		? item->unread(thread)
 		: item->isUnreadMedia();
 	if (unseen) {
 		return false;
 	}
 	const auto history = item->history();
 	const auto peer = history->peer;
+	if (const auto user = peer->asUser()) {
+		if (user->isSelf()
+			|| user->isBot()
+			|| user->isServiceUser()
+			|| user->readDatesPrivate()) {
+			return false;
+		}
+		const auto &appConfig = peer->session().appConfig();
+		const auto expirePeriod = appConfig.get<int>(
+			"pm_read_date_expire_period",
+			7 * 86400);
+		if (item->date() + int64(expirePeriod) <= int64(base::unixtime::now())) {
+			return false;
+		}
+		return true;
+	}
 	const auto chat = peer->asChat();
 	const auto megagroup = peer->asMegagroup();
-	if (!chat && !megagroup) {
+	if ((!chat && !megagroup)
+		|| (megagroup
+			&& (megagroup->flags() & ChannelDataFlag::ParticipantsHidden))
+		|| (megagroup && megagroup->isMonoforum())) {
 		return false;
 	}
-	const auto &appConfig = peer->session().account().appConfig();
-	const auto expirePeriod = TimeId(appConfig.get<double>(
+	const auto &appConfig = peer->session().appConfig();
+	const auto expirePeriod = appConfig.get<int>(
 		"chat_read_mark_expire_period",
-		7 * 86400.));
-	if (item->date() + expirePeriod <= base::unixtime::now()) {
+		7 * 86400);
+	if (item->date() + int64(expirePeriod) <= int64(base::unixtime::now())) {
 		return false;
 	}
-	const auto maxCount = int(appConfig.get<double>(
+	const auto maxCount = appConfig.get<int>(
 		"chat_read_mark_size_threshold",
-		50));
+		50);
 	const auto count = megagroup ? megagroup->membersCount() : chat->count;
 	if (count <= 0 || count > maxCount) {
 		return false;
@@ -579,8 +733,13 @@ bool WhoReadExists(not_null<HistoryItem*> item) {
 	return true;
 }
 
-bool WhoReactedExists(not_null<HistoryItem*> item) {
-	return item->canViewReactions() || WhoReadExists(item);
+bool WhoReactedExists(
+		not_null<HistoryItem*> item,
+		WhoReactedList list) {
+	if (item->canViewReactions() || WhoReadExists(item)) {
+		return true;
+	}
+	return (list == WhoReactedList::One) && item->history()->peer->isUser();
 }
 
 rpl::producer<Ui::WhoReadContent> WhoReacted(
@@ -588,15 +747,42 @@ rpl::producer<Ui::WhoReadContent> WhoReacted(
 		not_null<QWidget*> context,
 		const style::WhoRead &st,
 		std::shared_ptr<WhoReadList> whoReadIds) {
-	return WhoReacted(item, QString(), context, st, std::move(whoReadIds));
+	return WhoReacted(item, {}, context, st, std::move(whoReadIds));
 }
 
 rpl::producer<Ui::WhoReadContent> WhoReacted(
 		not_null<HistoryItem*> item,
-		const QString &reaction,
+		const Data::ReactionId &reaction,
 		not_null<QWidget*> context,
 		const style::WhoRead &st) {
 	return WhoReacted(item, reaction, context, st, nullptr);
 }
 
+[[nodiscard]] rpl::producer<Ui::WhoReadContent> WhenDate(
+		not_null<PeerData*> author,
+		TimeId date,
+		Ui::WhoReadType type) {
+	return rpl::single(Ui::WhoReadContent{
+		.participants = { Ui::WhoReadParticipant{
+			.name = author->name(),
+			.date = FormatReadDate(date, QDateTime::currentDateTime()),
+			.id = author->id.value,
+		} },
+		.type = type,
+		.fullReadCount = 1,
+	});
+}
+
+rpl::producer<Ui::WhoReadContent> WhenEdited(
+		not_null<PeerData*> author,
+		TimeId date) {
+	return WhenDate(author, date, Ui::WhoReadType::Edited);
+}
+
+rpl::producer<Ui::WhoReadContent> WhenOriginal(
+		not_null<PeerData*> author,
+		TimeId date) {
+	return WhenDate(author, date, Ui::WhoReadType::Original);
+}
+
 } // namespace Api
