@@ -69,12 +69,6 @@ constexpr auto kStarsStatsShortPollDelay = 30 * crl::time(1000);
 	return PinFinishDate(message.peer, message.date, message.stars);
 }
 
-[[nodiscard]] std::optional<PeerId> MaybeShownPeer(
-		uint32 privacySet,
-		PeerId shownPeer) {
-	return privacySet ? shownPeer : std::optional<PeerId>();
-}
-
 } // namespace
 
 Messages::Messages(not_null<GroupCall*> call, not_null<MTP::Sender*> api)
@@ -104,9 +98,7 @@ Messages::~Messages() {
 		finishPaidSending({
 			.count = int(_paid.sending),
 			.valid = true,
-			.shownPeer = MaybeShownPeer(
-				_paid.sendingPrivacySet,
-				_paid.sendingShownPeer),
+			.shownPeer = _paid.sendingShownPeer,
 		}, false);
 	}
 }
@@ -164,6 +156,7 @@ void Messages::send(TextWithTags text, int stars) {
 	_sendingIdByRandomId.emplace(randomId, localId);
 
 	const auto from = _call->messagesFrom();
+	const auto creator = _real->creator();
 	const auto skip = skipMessage(prepared, stars);
 	if (skip) {
 		_skippedIds.emplace(localId);
@@ -173,7 +166,7 @@ void Messages::send(TextWithTags text, int stars) {
 			.peer = from,
 			.text = std::move(prepared),
 			.stars = stars,
-			.admin = (from == _call->peer()),
+			.admin = (from == _call->peer()) || (creator && from->isSelf()),
 			.mine = true,
 		});
 	}
@@ -233,7 +226,8 @@ void Messages::received(const MTPDupdateGroupCallMessage &data) {
 		fields.vfrom_id(),
 		fields.vmessage(),
 		fields.vdate().v,
-		fields.vpaid_message_stars().value_or_empty());
+		fields.vpaid_message_stars().value_or_empty(),
+		fields.is_from_admin());
 }
 
 void Messages::received(const MTPDupdateGroupCallEncryptedMessage &data) {
@@ -268,6 +262,7 @@ void Messages::received(const MTPDupdateGroupCallEncryptedMessage &data) {
 		deserialized->message,
 		base::unixtime::now(), // date
 		0, // stars
+		false,
 		true); // checkCustomEmoji
 }
 
@@ -332,6 +327,7 @@ void Messages::received(
 		const MTPTextWithEntities &message,
 		TimeId date,
 		int stars,
+		bool fromAdmin,
 		bool checkCustomEmoji) {
 	const auto peer = _call->peer();
 	const auto i = ranges::find(_messages, id, &Message::id);
@@ -381,7 +377,7 @@ void Messages::received(
 			.peer = author,
 			.text = std::move(text),
 			.stars = stars,
-			.admin = (author == _call->peer()),
+			.admin = fromAdmin,
 			.mine = mine,
 		});
 		ranges::sort(_messages, ranges::less(), &Message::id);
@@ -513,26 +509,23 @@ PeerId Messages::reactionsLocalShownPeer() const {
 				return entry.peer ? entry.peer->id : PeerId();
 			}
 		}
-		return _session->userPeerId();
+		return _call->messagesFrom()->id;
 		//const auto api = &_session->api();
 		//return api->globalPrivacy().paidReactionShownPeerCurrent();
 	};
-	return (_paid.scheduledFlag && _paid.scheduledPrivacySet)
+	return _paid.scheduledFlag
 		? _paid.scheduledShownPeer
-		: (_paid.sendingFlag && _paid.sendingPrivacySet)
+		: _paid.sendingFlag
 		? _paid.sendingShownPeer
 		: minePaidShownPeer();
 }
 
-void Messages::reactionsPaidAdd(int count, std::optional<PeerId> shownPeer) {
+void Messages::reactionsPaidAdd(int count) {
 	Expects(count >= 0);
 
 	_paid.scheduled += count;
 	_paid.scheduledFlag = 1;
-	if (shownPeer.has_value()) {
-		_paid.scheduledShownPeer = *shownPeer;
-		_paid.scheduledPrivacySet = true;
-	}
+	_paid.scheduledShownPeer = _call->messagesFrom()->id;
 	if (count > 0) {
 		_session->credits().lock(CreditsAmount(count));
 	}
@@ -551,7 +544,6 @@ void Messages::reactionsPaidScheduledCancel() {
 	_paid.scheduled = 0;
 	_paid.scheduledFlag = 0;
 	_paid.scheduledShownPeer = 0;
-	_paid.scheduledPrivacySet = 0;
 	_paidChanges.fire({});
 }
 
@@ -566,17 +558,13 @@ Data::PaidReactionSend Messages::startPaidReactionSending() {
 	_paid.sending = _paid.scheduled;
 	_paid.sendingFlag = _paid.scheduledFlag;
 	_paid.sendingShownPeer = _paid.scheduledShownPeer;
-	_paid.sendingPrivacySet = _paid.scheduledPrivacySet;
 	_paid.scheduled = 0;
 	_paid.scheduledFlag = 0;
 	_paid.scheduledShownPeer = 0;
-	_paid.scheduledPrivacySet = 0;
 	return {
 		.count = int(_paid.sending),
 		.valid = true,
-		.shownPeer = MaybeShownPeer(
-			_paid.sendingPrivacySet,
-			_paid.sendingShownPeer),
+		.shownPeer = _paid.sendingShownPeer,
 	};
 }
 
@@ -585,25 +573,24 @@ void Messages::finishPaidSending(
 		bool success) {
 	Expects(send.count == _paid.sending);
 	Expects(send.valid == (_paid.sendingFlag == 1));
-	Expects(send.shownPeer == MaybeShownPeer(
-		_paid.sendingPrivacySet,
-		_paid.sendingShownPeer));
+	Expects(send.shownPeer == _paid.sendingShownPeer);
 
 	_paid.sending = 0;
 	_paid.sendingFlag = 0;
 	_paid.sendingShownPeer = 0;
-	_paid.sendingPrivacySet = 0;
 	if (const auto amount = send.count) {
 		if (success) {
+			const auto from = _session->data().peer(*send.shownPeer);
 			_session->credits().withdrawLocked(CreditsAmount(amount));
 
 			auto &donors = _paid.top.topDonors;
-			const auto i = ranges::find(donors, true, &StarsTopDonor::my);
+			const auto i = ranges::find(donors, true, &StarsDonor::my);
 			if (i != end(donors)) {
+				i->peer = from;
 				i->stars += amount;
 			} else {
 				donors.push_back({
-					.peer = _session->user(),
+					.peer = from,
 					.stars = amount,
 					.my = true,
 				});
@@ -628,7 +615,7 @@ void Messages::reactionsPaidSend() {
 	const auto randomId = base::RandomValue<uint64>();
 	_sendingIdByRandomId.emplace(randomId, localId);
 
-	const auto from = _call->messagesFrom();
+	const auto from = _session->data().peer(*send.shownPeer);
 	const auto stars = int(send.count);
 	const auto skip = skipMessage({}, stars);
 	if (skip) {
@@ -672,7 +659,7 @@ void Messages::undoScheduledPaidOnDestroy() {
 
 Messages::PaidLocalState Messages::starsLocalState() const {
 	const auto &donors = _paid.top.topDonors;
-	const auto i = ranges::find(donors, true, &StarsTopDonor::my);
+	const auto i = ranges::find(donors, true, &StarsDonor::my);
 	const auto local = int(_paid.scheduled);
 	const auto my = (i != end(donors) ? i->stars : 0) + local;
 	const auto total = _paid.top.total + local;
@@ -724,7 +711,7 @@ void Messages::addStars(not_null<PeerData*> from, int stars, bool mine) {
 	const auto i = ranges::find(
 		_paid.top.topDonors,
 		from.get(),
-		&StarsTopDonor::peer);
+		&StarsDonor::peer);
 	if (i != end(_paid.top.topDonors)) {
 		i->stars += stars;
 	} else {
@@ -737,8 +724,8 @@ void Messages::addStars(not_null<PeerData*> from, int stars, bool mine) {
 	ranges::stable_sort(
 		_paid.top.topDonors,
 		ranges::greater(),
-		&StarsTopDonor::stars);
-	_paidChanges.fire({});
+		&StarsDonor::stars);
+	_paidChanges.fire({ .peer = from, .stars = stars });
 }
 
 } // namespace Calls::Group
