@@ -65,14 +65,18 @@ Viewport::Viewport(
 }
 
 Viewport::~Viewport() {
-	if (_borrowed && _opengl) {
-		const auto w = static_cast<QOpenGLWidget*>(widget().get());
-		w->makeCurrent();
-		const auto context = w->context();
-		const auto valid = w->isValid()
-			&& context
-			&& (QOpenGLContext::currentContext() == context);
-		ensureBorrowedCleared(valid ? context->functions() : nullptr);
+	if (_borrowed) {
+		if (_opengl) {
+			const auto w = static_cast<QOpenGLWidget*>(widget().get());
+			w->makeCurrent();
+			const auto context = w->context();
+			const auto valid = w->isValid()
+				&& context
+				&& (QOpenGLContext::currentContext() == context);
+			ensureBorrowedCleared(valid ? context->functions() : nullptr);
+		} else {
+			ensureBorrowedCleared();
+		}
 	}
 }
 
@@ -787,13 +791,13 @@ void Viewport::updateTilesGeometryColumn(int outerWidth) {
 			top += height + st::groupCallVideoSmallSkip;
 		}
 	};
-	const auto topPeer = _large ? _large->row()->peer().get() : nullptr;
+	const auto topPeer = _large ? _large->peer().get() : nullptr;
 	const auto reorderNeeded = [&] {
 		if (!topPeer) {
 			return false;
 		}
 		for (const auto &tile : _tiles) {
-			if (tile.get() != _large && tile->row()->peer() == topPeer) {
+			if (tile.get() != _large && tile->peer() == topPeer) {
 				return (tile.get() != _tiles.front().get())
 					&& !tile->trackOrUserpicSize().isEmpty();
 			}
@@ -809,7 +813,7 @@ void Viewport::updateTilesGeometryColumn(int outerWidth) {
 		ranges::stable_partition(
 			_tilesForOrder,
 			[&](not_null<VideoTile*> tile) {
-				return (tile->row()->peer() == topPeer);
+				return (tile->peer() == topPeer);
 			});
 		for (const auto &tile : _tilesForOrder) {
 			layoutNext(tile);
@@ -925,6 +929,7 @@ rpl::producer<bool> Viewport::mouseInsideValue() const {
 
 void Viewport::ensureBorrowedRenderer(QOpenGLFunctions &f) {
 	Expects(_borrowed != nullptr);
+	Expects(_opengl);
 
 	if (_borrowedRenderer) {
 		return;
@@ -935,6 +940,7 @@ void Viewport::ensureBorrowedRenderer(QOpenGLFunctions &f) {
 
 void Viewport::ensureBorrowedCleared(QOpenGLFunctions *f) {
 	Expects(_borrowed != nullptr);
+	Expects(_opengl);
 
 	if (const auto renderer = base::take(_borrowedRenderer)) {
 		renderer->deinit(f);
@@ -948,6 +954,23 @@ void Viewport::borrowedPaint(QOpenGLFunctions &f) {
 	_borrowedRenderer->paint(static_cast<QOpenGLWidget*>(widget().get()), f);
 }
 
+void Viewport::ensureBorrowedRenderer() {
+	Expects(_borrowed != nullptr);
+	Expects(!_opengl);
+
+	if (_borrowedRenderer) {
+		return;
+	}
+	_borrowedRenderer = makeRenderer();
+}
+
+void Viewport::ensureBorrowedCleared() {
+	Expects(_borrowed != nullptr);
+	Expects(!_opengl);
+
+	base::take(_borrowedRenderer);
+}
+
 void Viewport::borrowedPaint(Painter &p, const QRegion &clip) {
 	Expects(_borrowedRenderer != nullptr);
 	Expects(!_opengl);
