@@ -149,6 +149,7 @@ struct GiftCode {
 	PeerData *stargiftReleasedBy = nullptr;
 	std::shared_ptr<UniqueGift> unique;
 	TextWithEntities message;
+	PeerData *auctionTo = nullptr;
 	ChannelData *channel = nullptr;
 	PeerData *channelFrom = nullptr;
 	uint64 channelSavedId = 0;
@@ -159,6 +160,7 @@ struct GiftCode {
 	int starsToUpgrade = 0;
 	int starsUpgradedBySender = 0;
 	int starsForDetailsRemove = 0;
+	int starsBid = 0;
 	int limitedCount = 0;
 	int limitedLeft = 0;
 	int64 count = 0;
@@ -166,6 +168,7 @@ struct GiftCode {
 	bool viaGiveaway : 1 = false;
 	bool transferred : 1 = false;
 	bool upgradeSeparate : 1 = false;
+	bool upgradeGifted : 1 = false;
 	bool upgradable : 1 = false;
 	bool unclaimed : 1 = false;
 	bool anonymous : 1 = false;
