@@ -25,6 +25,7 @@ enum class ChatRestriction;
 namespace Ui {
 class EmptyUserpic;
 struct BotVerifyDetails;
+struct ColorCollectible;
 } // namespace Ui
 
 namespace Main {
@@ -142,6 +143,9 @@ struct AllowedReactions {
 	not_null<Session*> owner,
 	const MTPInputUser &input);
 
+[[nodiscard]] Ui::ColorCollectible ParseColorCollectible(
+	const MTPDpeerColorCollectible &data);
+
 } // namespace Data
 
 class PeerClickHandler : public ClickHandler {
@@ -214,11 +218,33 @@ public:
 	[[nodiscard]] uint8 colorIndex() const {
 		return _colorIndex;
 	}
+	[[nodiscard]] auto colorCollectible() const
+	-> const std::shared_ptr<Ui::ColorCollectible> & {
+		return _colorCollectible;
+	}
+	bool changeColorCollectible(Ui::ColorCollectible data);
+	bool clearColorCollectible();
 	bool changeColorIndex(uint8 index);
 	bool clearColorIndex();
 	[[nodiscard]] DocumentId backgroundEmojiId() const;
 	bool changeBackgroundEmojiId(DocumentId id);
 
+	[[nodiscard]] std::optional<uint8> colorProfileIndex() const {
+		return _colorProfileIndex;
+	}
+	[[nodiscard]] auto colorProfileCollectible() const
+	-> const std::shared_ptr<Ui::ColorCollectible> & {
+		return _colorProfileCollectible;
+	}
+	bool changeColorProfileCollectible(Ui::ColorCollectible data);
+	bool changeColorProfileCollectible(
+		const tl::conditional<MTPPeerColor> &cloudColor);
+	bool clearColorProfileCollectible();
+	bool changeColorProfileIndex(uint8 index);
+	bool clearColorProfileIndex();
+	[[nodiscard]] DocumentId profileBackgroundEmojiId() const;
+	bool changeProfileBackgroundEmojiId(DocumentId id);
+
 	void setEmojiStatus(const MTPEmojiStatus &status);
 	void setEmojiStatus(EmojiStatusId emojiStatusId, TimeId until = 0);
 	[[nodiscard]] EmojiStatusId emojiStatusId() const;
@@ -271,6 +297,10 @@ public:
 	[[nodiscard]] Data::SavedSublist *monoforumSublistFor(
 		PeerId sublistPeerId) const;
 
+	[[nodiscard]] bool useSubsectionTabs() const;
+	[[nodiscard]] bool viewForumAsMessages() const;
+	void processTopics(const MTPVector<MTPForumTopic> &topics);
+
 	[[nodiscard]] Data::PeerNotifySettings &notify() {
 		return _notify;
 	}
@@ -357,11 +387,11 @@ public:
 	void setUserpicPhoto(const MTPPhoto &data);
 
 	void paintUserpic(
-		Painter &p,
+		QPainter &p,
 		Ui::PeerUserpicView &view,
 		PaintUserpicContext context) const;
 	void paintUserpic(
-			Painter &p,
+			QPainter &p,
 			Ui::PeerUserpicView &view,
 			int x,
 			int y,
@@ -376,7 +406,7 @@ public:
 		});
 	}
 	void paintUserpicLeft(
-			Painter &p,
+			QPainter &p,
 			Ui::PeerUserpicView &view,
 			int x,
 			int y,
@@ -485,10 +515,12 @@ public:
 	void saveTranslationDisabled(bool disabled);
 
 	void setBarSettings(const MTPPeerSettings &data);
-	bool changeColorIndex(const tl::conditional<MTPint> &cloudColorIndex);
 	bool changeBackgroundEmojiId(
 		const tl::conditional<MTPlong> &cloudBackgroundEmoji);
+	bool changeColorCollectible(
+		const tl::conditional<MTPPeerColor> &cloudColor);
 	bool changeColor(const tl::conditional<MTPPeerColor> &cloudColor);
+	bool changeColorProfile(const tl::conditional<MTPPeerColor> &cloudColor);
 
 	enum class BlockStatus : char {
 		Unknown,
@@ -543,9 +575,11 @@ public:
 		None,
 		HasRead,
 		HasUnread,
+		HasVideoStream,
 	};
 	[[nodiscard]] bool hasActiveStories() const;
 	[[nodiscard]] bool hasUnreadStories() const;
+	[[nodiscard]] bool hasActiveVideoStream() const;
 	void setStoriesState(StoriesState state);
 
 	[[nodiscard]] int peerGiftsCount() const;
@@ -593,6 +627,7 @@ private:
 
 	EmojiStatusId _emojiStatusId;
 	DocumentId _backgroundEmojiId = 0;
+	DocumentId _profileBackgroundEmojiId = 0;
 	crl::time _lastFullUpdate = 0;
 
 	QString _name;
@@ -605,12 +640,15 @@ private:
 
 	BarSettings _barSettings = PeerBarSettings(PeerBarSetting::Unknown);
 	std::unique_ptr<PeerBarDetails> _barDetails;
+	std::shared_ptr<Ui::ColorCollectible> _colorCollectible;
+	std::shared_ptr<Ui::ColorCollectible> _colorProfileCollectible;
 
 	BlockStatus _blockStatus = BlockStatus::Unknown;
 	LoadedStatus _loadedStatus = LoadedStatus::Not;
 	TranslationFlag _translationFlag = TranslationFlag::Unknown;
 	uint8 _colorIndex : 6 = 0;
 	uint8 _colorIndexCloud : 1 = 0;
+	std::optional<uint8> _colorProfileIndex;
 	uint8 _userpicHasVideo : 1 = 0;
 
 	QString _about;
@@ -635,4 +673,7 @@ void SetTopPinnedMessageId(
 	PeerId monoforumPeerId,
 	PeerData *migrated = nullptr);
 
+[[nodiscard]] uint64 BackgroundEmojiIdFromColor(const MTPPeerColor *color);
+[[nodiscard]] uint8 ColorIndexFromColor(const MTPPeerColor *color);
+
 } // namespace Data
