@@ -27,6 +27,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "lottie/lottie_frame_generator.h"
 #include "ffmpeg/ffmpeg_frame_generator.h"
 #include "chat_helpers/stickers_lottie.h"
+#include "info/channel_statistics/earn/earn_icons.h"
 #include "storage/file_download.h" // kMaxFileInMemory
 #include "ui/chat/chats_filter_tag.h"
 #include "ui/effects/premium_stars_colored.h"
@@ -40,7 +41,6 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "apiwrap.h"
 #include "styles/style_chat.h"
 #include "styles/style_chat_helpers.h"
-#include "styles/style_credits.h" // giftBoxByStarsStyle
 
 namespace Data {
 namespace {
@@ -100,10 +100,6 @@ private:
 		: FrameSizeFromTag(tag);
 }
 
-[[nodiscard]] QString InternalPrefix() {
-	return u"internal:"_q;
-}
-
 [[nodiscard]] QString UserpicEmojiPrefix() {
 	return u"userpic:"_q;
 }
@@ -569,8 +565,6 @@ std::unique_ptr<Ui::Text::CustomEmoji> CustomEmojiManager::create(
 		const auto original = data.mid(ForceStaticPrefix().size());
 		return std::make_unique<Ui::Text::FirstFrameEmoji>(
 			create(original, std::move(update), tag, sizeOverride));
-	} else if (data.startsWith(InternalPrefix())) {
-		return internal(data);
 	} else if (data.startsWith(UserpicEmojiPrefix())) {
 		const auto ratio = style::DevicePixelRatio();
 		const auto size = EmojiSizeFromTag(tag) / ratio;
@@ -618,26 +612,6 @@ std::unique_ptr<Ui::Text::CustomEmoji> CustomEmojiManager::create(
 	});
 }
 
-std::unique_ptr<Ui::Text::CustomEmoji> CustomEmojiManager::internal(
-		QStringView data) {
-	const auto v = data.mid(InternalPrefix().size()).split(',');
-	if (v.size() != 5 && v.size() != 1) {
-		return nullptr;
-	}
-	const auto index = v[0].toInt();
-	Assert(index >= 0 && index < _internalEmoji.size());
-
-	auto &info = _internalEmoji[index];
-	const auto padding = (v.size() == 5)
-		? QMargins(v[1].toInt(), v[2].toInt(), v[3].toInt(), v[4].toInt())
-		: QMargins();
-	return std::make_unique<Ui::CustomEmoji::Internal>(
-		data.toString(),
-		info.image,
-		padding,
-		info.textColor);
-}
-
 std::unique_ptr<Ui::Text::CustomEmoji> CustomEmojiManager::userpic(
 		QStringView data,
 		Fn<void()> update,
@@ -905,15 +879,15 @@ void CustomEmojiManager::repaintLater(
 		not_null<Ui::CustomEmoji::Instance*> instance,
 		Ui::CustomEmoji::RepaintRequest request) {
 	auto &bunch = _repaints[request.duration];
-	if (bunch.when < request.when) {
-		if (bunch.when > 0) {
-			for (const auto &already : bunch.instances) {
-				if (already.get() == instance) {
-					// Still waiting for full bunch repaint, don't bump.
-					return;
-				}
+	if (bunch.when > 0) {
+		for (const auto &already : bunch.instances) {
+			if (already.get() == instance) {
+				// Still waiting for full bunch repaint, don't bump.
+				return;
 			}
 		}
+	}
+	if (bunch.when < request.when) {
 		bunch.when = request.when;
 #if 0 // inject-to-on_main
 		_repaintsLastAdded = request.when;
@@ -1020,57 +994,6 @@ uint64 CustomEmojiManager::coloredSetId() const {
 	return _coloredSetId;
 }
 
-TextWithEntities CustomEmojiManager::creditsEmoji(QMargins padding) {
-	return Ui::Text::SingleCustomEmoji(
-		registerInternalEmoji(
-			Ui::GenerateStars(st::normalFont->height, 1),
-			padding,
-			false));
-}
-
-TextWithEntities CustomEmojiManager::ministarEmoji(QMargins padding) {
-	return Ui::Text::SingleCustomEmoji(
-		registerInternalEmoji(
-			Ui::GenerateStars(st::giftBoxByStarsStyle.font->height, 1),
-			padding,
-			false));
-}
-
-QString CustomEmojiManager::registerInternalEmoji(
-		QImage emoji,
-		QMargins padding,
-		bool textColor) {
-	_internalEmoji.push_back({ std::move(emoji), textColor });
-	return InternalPrefix()
-		+ QString::number(_internalEmoji.size() - 1)
-		+ InternalPadding(padding);
-}
-
-QString CustomEmojiManager::registerInternalEmoji(
-		const style::icon &icon,
-		QMargins padding,
-		bool textColor) {
-	const auto i = _iconEmoji.find(&icon);
-	if (i != end(_iconEmoji)) {
-		return i->second + InternalPadding(padding);
-	}
-	auto image = QImage(
-		icon.size() * style::DevicePixelRatio(),
-		QImage::Format_ARGB32_Premultiplied);
-	image.fill(Qt::transparent);
-	image.setDevicePixelRatio(style::DevicePixelRatio());
-	auto p = QPainter(&image);
-	icon.paint(p, 0, 0, icon.width());
-	p.end();
-
-	const auto result = registerInternalEmoji(
-		std::move(image),
-		QMargins{},
-		textColor);
-	_iconEmoji.emplace(&icon, result);
-	return result + InternalPadding(padding);
-}
-
 [[nodiscard]] QString CustomEmojiManager::peerUserpicEmojiData(
 		not_null<PeerData*> peer,
 		QMargins padding,
