@@ -7,34 +7,38 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "history/view/history_view_subsection_tabs.h"
 
+#include "apiwrap.h"
 #include "base/qt/qt_key_modifiers.h"
 #include "core/ui_integration.h"
-#include "data/stickers/data_custom_emoji.h"
+#include "data/data_changes.h"
 #include "data/data_channel.h"
-#include "data/data_forum.h"
 #include "data/data_forum_topic.h"
+#include "data/data_forum.h"
 #include "data/data_saved_messages.h"
 #include "data/data_saved_sublist.h"
 #include "data/data_session.h"
 #include "data/data_thread.h"
 #include "data/data_user.h"
+#include "data/stickers/data_custom_emoji.h"
 #include "dialogs/dialogs_main_list.h"
 #include "history/history.h"
 #include "lang/lang_keys.h"
-#include "main/main_session.h"
 #include "main/main_session_settings.h"
+#include "main/main_session.h"
+#include "ui/controls/subsection_tabs_slider_reorder.h"
 #include "ui/controls/subsection_tabs_slider.h"
+#include "ui/dynamic_image.h"
+#include "ui/dynamic_thumbnails.h"
 #include "ui/effects/ripple_animation.h"
-#include "ui/widgets/menu/menu_add_action_callback_factory.h"
-#include "ui/widgets/menu/menu_add_action_callback.h"
 #include "ui/text/text_utilities.h"
+#include "ui/ui_utility.h"
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/discrete_sliders.h"
+#include "ui/widgets/menu/menu_add_action_callback_factory.h"
+#include "ui/widgets/menu/menu_add_action_callback.h"
 #include "ui/widgets/popup_menu.h"
 #include "ui/widgets/scroll_area.h"
 #include "ui/widgets/shadow.h"
-#include "ui/dynamic_image.h"
-#include "ui/dynamic_thumbnails.h"
 #include "window/window_peer_menu.h"
 #include "window/window_separate_id.h"
 #include "window/window_session_controller.h"
@@ -61,6 +65,10 @@ SubsectionTabs::SubsectionTabs(
 	refreshSlice();
 	setup(parent);
 
+	session().data().pinnedDialogsOrderUpdated() | rpl::start_with_next([=] {
+		_refreshed.fire({});
+	}, _lifetime);
+
 	dataChanged() | rpl::start_with_next([=] {
 		if (_loading) {
 			_loading = false;
@@ -112,6 +120,7 @@ void SubsectionTabs::setupHorizontal(not_null<QWidget*> parent) {
 	const auto shadow = Ui::CreateChild<Ui::PlainShadow>(_horizontal);
 	const auto slider = scroll->setOwnedWidget(
 		object_ptr<Ui::HorizontalSlider>(scroll));
+	_reorder = std::make_unique<Ui::SubsectionSliderReorder>(slider, scroll);
 	setupSlider(scroll, slider, false);
 
 	shadow->showOn(rpl::single(
@@ -184,6 +193,7 @@ void SubsectionTabs::setupVertical(not_null<QWidget*> parent) {
 	const auto shadow = Ui::CreateChild<Ui::PlainShadow>(_vertical);
 	const auto slider = scroll->setOwnedWidget(
 		object_ptr<Ui::VerticalSlider>(scroll));
+	_reorder = std::make_unique<Ui::SubsectionSliderReorder>(slider, scroll);
 	setupSlider(scroll, slider, true);
 
 	shadow->showOn(rpl::single(
@@ -219,6 +229,9 @@ void SubsectionTabs::setupSlider(
 		not_null<Ui::SubsectionSlider*> slider,
 		bool vertical) {
 	slider->sectionActivated() | rpl::start_with_next([=](int active) {
+		if (_reordering) {
+			return;
+		}
 		const auto newWindow = base::IsCtrlPressed();
 		if (active >= 0
 			&& active < _slice.size()
@@ -234,8 +247,29 @@ void SubsectionTabs::setupSlider(
 				_controller->showThread(thread, ShowAtUnreadMsgId, params);
 			}
 		}
+		_reorder->finishReordering();
 	}, slider->lifetime());
 
+	_reorder->updates(
+	) | rpl::start_with_next([=](Ui::SubsectionSliderReorder::Single data) {
+		using State = Ui::SubsectionSliderReorder::State;
+		if (data.state == State::Started) {
+			++_reordering;
+		} else {
+			Ui::PostponeCall(slider, [=] {
+				--_reordering;
+			});
+			if (data.state == State::Applied) {
+				applyReorder(data.widget, data.oldPosition, data.newPosition);
+				slider->recalculatePinnedPositions();
+			}
+		}
+	}, slider->lifetime());
+
+	slider->setIsReorderingCallback([=] {
+		return _reordering > 0;
+	});
+
 	slider->sectionContextMenu() | rpl::start_with_next([=](int index) {
 		if (index >= 0 && index < _slice.size()) {
 			showThreadContextMenu(_slice[index].thread);
@@ -311,6 +345,19 @@ void SubsectionTabs::startScrollChecking(
 			refreshAroundMiddle(scroll, slider);
 		}
 	}, scroll->lifetime());
+
+	session().changes().peerUpdates(
+		Data::PeerUpdate::Flag::Rights
+	) | rpl::filter([=](const Data::PeerUpdate &update) {
+		return (update.peer == _history->peer);
+	}) | rpl::start_with_next([=] {
+		if (_reorder) {
+			_reorder->cancel();
+			if (_history->peer->canManageTopics()) {
+				_reorder->start();
+			}
+		}
+	}, _lifetime);
 }
 
 void SubsectionTabs::startFillingSlider(
@@ -336,11 +383,20 @@ void SubsectionTabs::startFillingSlider(
 		auto updated = Cache();
 		auto sections = std::vector<Ui::SubsectionTab>();
 		auto activeIndex = -1;
+		auto fixedCount = 1; // 1 is the first button.
+		auto pinnedCount = 0;
 		for (const auto &item : _slice) {
 			const auto index = int(sections.size());
 			if (item.thread == _active) {
 				activeIndex = index;
 			}
+			if (item.thread->fixedOnTopIndex()) {
+				++fixedCount;
+			}
+			if (item.thread->isPinnedDialog(FilterId())
+				&& item.thread->asTopic()) {
+				++pinnedCount;
+			}
 			const auto textFg = [=] {
 				return anim::color(
 					st::windowSubTextFg,
@@ -397,11 +453,23 @@ void SubsectionTabs::startFillingSlider(
 						).append(' ').append(peer->shortName()),
 					});
 				}
+			} else if (item.thread->peer()->isBot()) {
+				sections.push_back({
+					.text = { tr::lng_bot_new_chat(tr::now) },
+				});
+				if (vertical) {
+					auto &last = sections.back();
+					last.userpic = Ui::MakeNewChatSubsectionsThumbnail(
+						textFg);
+				}
 			} else {
 				sections.push_back({
 					.text = { tr::lng_filters_all_short(tr::now) },
-					.userpic = Ui::MakeAllSubsectionsThumbnail(textFg),
 				});
+				if (vertical) {
+					auto &last = sections.back();
+					last.userpic = Ui::MakeAllSubsectionsThumbnail(textFg);
+				}
 			}
 			auto &section = sections.back();
 			section.badges = item.badges;
@@ -457,13 +525,27 @@ void SubsectionTabs::startFillingSlider(
 			.context = Core::TextContext({
 				.session = &session(),
 			}),
+			.fixed = fixedCount,
+			.pinned = pinnedCount,
 		}, paused);
+		_reorder->clearPinnedIntervals();
+		_reorder->addPinnedInterval(0, 1);
+		if (pinnedCount > 1) {
+			const auto from = 1 + pinnedCount;
+			_reorder->addPinnedInterval(from, slider->sectionsCount() - from);
+		}
 
 		const auto ignoreActiveScroll = (scrollSavingIndex >= 0);
 		slider->setActiveSectionFast(activeIndex, ignoreActiveScroll);
 
 		_sectionsSlice = _slice;
 		Assert(slider->sectionsCount() == _slice.size());
+
+		_reorder->cancel();
+		if ((pinnedCount > 1) && _history->peer->canManageTopics()) {
+			_reorder->start();
+		}
+
 		if (ignoreActiveScroll) {
 			Assert(scrollSavingIndex < slider->sectionsCount());
 			const auto position = scrollSavingShift
@@ -658,6 +740,16 @@ void SubsectionTabs::track() {
 		}) | rpl::start_with_next([=] {
 			scheduleRefresh();
 		}, _lifetime);
+
+		forum->session().changes().topicUpdates(
+			Data::TopicUpdate::Flag::Title
+			| Data::TopicUpdate::Flag::IconId
+			| Data::TopicUpdate::Flag::ColorId
+		) | rpl::filter([=](const Data::TopicUpdate &update) {
+			return update.topic->forum() == forum;
+		}) | rpl::start_with_next([=] {
+			scheduleRefresh();
+		}, _lifetime);
 	} else if (const auto monoforum = _history->peer->monoforum()) {
 		monoforum->sublistDestroyed(
 		) | rpl::start_with_next([=](not_null<Data::SavedSublist*> sublist) {
@@ -840,11 +932,35 @@ bool SubsectionTabs::switchTo(
 
 bool SubsectionTabs::UsedFor(not_null<Data::Thread*> thread) {
 	const auto history = thread->owningHistory();
-	if (history->amMonoforumAdmin()) {
-		return true;
+	return history->amMonoforumAdmin()
+		|| history->peer->useSubsectionTabs();
+}
+
+void SubsectionTabs::applyReorder(
+		not_null<Ui::RpWidget*> widget,
+		int oldPosition,
+		int newPosition) {
+	if (newPosition == oldPosition) {
+		return;
+	}
+
+	Assert(oldPosition >= 0 && oldPosition < _slice.size());
+	Assert(newPosition >= 0 && newPosition < _slice.size());
+
+	base::reorder(_slice, oldPosition, newPosition);
+
+	if (const auto forum = _history->asForum()) {
+		auto topicIds = QVector<MTPint>();
+		for (const auto &item : _slice) {
+			if (item.thread->isPinnedDialog(FilterId())) {
+				if (const auto topic = item.thread->asTopic()) {
+					topicIds.push_back(MTP_int(topic->rootId()));
+				}
+			}
+		}
+		forum->topicsList()->pinned()->applyList(forum, topicIds);
+		session().api().savePinnedOrder(forum);
 	}
-	const auto channel = history->peer->asChannel();
-	return channel && channel->useSubsectionTabs();
 }
 
 } // namespace HistoryView
