@@ -32,6 +32,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "boxes/add_contact_box.h"
 #include "boxes/premium_limits_box.h"
 #include "boxes/username_box.h"
+#include "boxes/peers/edit_peer_color_box.h"
 #include "data/data_session.h"
 #include "data/data_user.h"
 #include "data/data_peer_values.h"
@@ -441,6 +442,12 @@ void SetupPersonalChannel(
 		edit,
 		{ &st::menuIconChannel });
 
+	AddPeerColorButton(
+		container,
+		controller->uiShow(),
+		self,
+		st::settingsColorButton);
+
 	Ui::AddSkip(container);
 	Ui::AddDivider(container);
 }
